#!/bin/sh
# ChessDB (based on Scid)
#

#
# This is freely redistributable software; see the file named "COPYING"
# or "copying.txt" that came with this program.
#
# To contact the maintainer of ChessDB, email drkirkby@users.sourceforge.net
# ChessDB is based heavily on Scid, the author of which is 
# Shane Hudson. His email is sgh@users.sourceforge.net

#
# The following few comments are only for Unix versions of ChessDB:
#

# The "\" at the end of the comment line below is necessary! It means
#   that the "exec" line is a comment to Tcl/Tk, but not to /bin/sh.
# The next line restarts using tkchessdb: \
exec tkchessdb "$0" "$@"

# For the above to work, tkchessdb must be in a directory in your PATH.
# Alternatively, you can set the first line to start tkchessdb directly
# by specifying the full name of tkchessdb, eg:
# #!/home/myname/bin/tkchessdb

############################################################

set chessdbVersion "3.6"


# Determine operating system platform: unix or windows
#

set DownloadingNow 0
set windowsOS 0
if {$tcl_platform(platform) == "windows"} { set windowsOS 1 }
set unixOS 0
if {$tcl_platform(platform) == "unix"} { set unixOS 1 }

# A lot of code assumes tcl_platform is either windows or unix, so
# lotsa stuff may break if this is not the case.
#
# if {(! $windowsOS)  &&  (! $unixOS)} {
  # What to do here? Warn the user...?
# }


# Check that on Unix, the version of tkchessdb matches the version of this
# script or on Windows, that the chessdb.exe and chessdb.gui versions are identical.
#
if {[string compare [sc_info version] $chessdbVersion]} {
  wm withdraw .
  if {$windowsOS} {
    set msg "This is ChessDB version [sc_info version], but the chessdb.gui data\n"
    append msg "file has the version number $chessdbVersion.\n"
  } else {
    set msg "This program,\n\"$argv0\",\nis version $chessdbVersion,\nbut the "
    append msg "tkchessdb program \nit uses is version [sc_info version].\n"
    append msg "Check that the path to tkchessdb is correct." 
  }
  tk_messageBox -type ok -icon error -title "ChessDB: Version Error" -message $msg
  exit 1
}

# Alter the version if any patches have been made to the Tcl code only:
set majorVersion 3
set minorVersion 6
set patchlevel   18
set betalevel    0
set releaseyear       2007
set releasemonth         4
set releaseday           22
if { $betalevel >= 1 } {
   set chessdbVersion [format "%d.%d.%d-beta-%d" $majorVersion $minorVersion $patchlevel $betalevel]
} {
   set chessdbVersion [format "%d.%d.%d" $majorVersion $minorVersion $patchlevel]
}
set chessdbVersionDate [ format  "%04d-%02d-%02d"  $releaseyear $releasemonth $releaseday]
set ::ChessDBVersionDate [ format  "%04d-%02d-%02d"  $releaseyear $releasemonth $releaseday]
set ::ChessDBVersionNumber $chessdbVersion

#############################################################
#
# NAMESPACES
#
# The main Tcl/Tk namespaces used in the ChessDB application are
# initialized here, so that default values can be set up and
# altered when the user options file is loaded.
#
foreach ns {
  ::splash
  ::utils
    ::utils::date ::utils::font ::utils::history ::utils::pane ::utils::string
    ::utils::sound ::utils::validate ::utils::win ::utils::download
  ::file
    ::file::finder ::file::maint ::maint
    ::bookmarks
  ::edit
  ::game
    ::gbrowser
  ::search
    ::search::filter ::search::board ::search::header ::seach::material
  ::windows
    ::windows::gamelist ::windows::stats ::tree ::windows::tree
    ::windows::switcher ::windows::eco ::crosstab ::pgn
  ::tools
    ::tools::analysis ::tools::email
    ::tools::graphs
      ::tools::graphs::filter ::tools::graphs::rating ::tools::graphs::score
    ::tb ::optable
    ::tools::twic  
    ::tools::downloadFromChessServer
  ::help
  ::board ::move
} {
  namespace eval $ns {}
}


#############################################################
# Customisable variables:

# chessdbExeDir: contains the directory of the ChessDB executable program.
# Not used under Linux/Unix, but is used under Windows to determine the
# directory where the options file is stored.
set chessdbExeDir [file dirname [info nameofexecutable]]

# chessdbUserDir: location of user-specific ChessDB files.
# This is "~/.chessdb" on Unix, and the ChessDB exectuable dir on Windows.
if {$windowsOS} {
  set chessdbUserDir $chessdbExeDir
} else {
  set chessdbUserDir [file nativename "~/.chessdb"]
}

# chessdbConfigDir, chessdbDataDir, chessdbLogDir:
#   Location of ChessDB configuration, data and log files.
set chessdbConfigDir [file nativename [file join $chessdbUserDir "config"]]
set chessdbDataDir [file nativename [file join $chessdbUserDir "data"]]
set chessdbLogDir [file nativename [file join $chessdbUserDir "log"]]
set chessdbTWICDir [file nativename [file join $chessdbUserDir "twic"]]

# ecoFile: the ECO file for opening classification. ChessDB will try to load
# this first, and if that fails, it will try to load  "chessdb.eco" in the
# current directory.
if {$windowsOS} {
  set ecoFile [file join $chessdbDataDir "chessdb.eco"]
} else {
  set ecoFile "/usr/local/share/chessdb/chessdb.eco"
}

if {$windowsOS} {
  set defaultTBDIR [file nativename [file join $::chessdbExeDir "tablebases"]]
} else {
  set defaultTBDIR "/usr/local/share/chessdb/tablebases"
}

# boardSizes: a list of the available board sizes.
set boardSizes [list 25 30 35 40 45 50 55 60 65 70 75 80 85 90 95]
set boardSizesOLD [list 21 25 29 33 37 40 45 49 54 58 64 72]

# boardSize: Default board size. See the available board sizes above.
set boardSize 40

# boardStyle: Default board piece set. See bitmaps.tcl for styles.
set boardStyle Alpha

# language for help pages and messages:
set language E
set oldLang X

# Toolbar configuration:
foreach {tbicon status}  {
  new 1 open 1 save 1 close 1
  finder 1 bkm 1 gprev 1 gnext 1
  cut 0 copy 0 paste 0
  rfilter 1 bsearch 1 hsearch 1 msearch 1
  switcher 1 glist 1 pgn 1 tmt 1 maint 1 eco 1 tree 1 crosst 1 engine 1
} {
  set toolbar($tbicon) $status
}

# boardCoords: 1 to show board Coordinates, 0 to hide them.
set boardCoords 0

# boardSTM: 1 to show side-to-move icon, 0 to hide it.
set boardSTM 1

# Default values for fonts:
proc createFont {name} {
  set opts $::fontOptions($name)
  font create font_$name \
    -family [lindex $opts 0] -size [lindex $opts 1] \
    -weight [lindex $opts 2] -slant [lindex $opts 3]
}

proc configureFont {name} {
  set opts $::fontOptions($name)
  font configure font_$name \
    -family [lindex $opts 0] -size [lindex $opts 1] \
    -weight [lindex $opts 2] -slant [lindex $opts 3]
}

if {$windowsOS} {
  set fontOptions(Regular) [list Arial           10 normal roman]
  set fontOptions(Menu)    [list {MS Sans Serif}  9 normal roman]
  set fontOptions(Small)   [list Arial            9 normal roman]
  set fontOptions(Tiny)    [list Arial            8 normal roman]
  set fontOptions(Fixed)   [list Courier          9 normal roman]
} else {
  set fontOptions(Regular) [list helvetica 11 normal roman]
  set fontOptions(Menu)    [list helvetica 10 normal roman]
  set fontOptions(Small)   [list helvetica 10 normal roman]
  set fontOptions(Tiny)    [list helvetica  9 normal roman]
  set fontOptions(Fixed)   [list fixed     10 normal roman]
}

createFont Regular
createFont Menu
createFont Small
createFont Tiny
createFont Fixed

# Analysis command: to start chess analysis engine.
set analysisCommand ""
if {$windowsOS} {
  set analysisChoices {crafty.exe}
} else {
  set analysisChoices {crafty}
}

# Colors: dark and lite are square colors
#     whitecolor/blackcolor are piece colors
#     highcolor is the color when something is selected.
#     bestcolor is used to indicate a suggested move square.
set dark        "\#70a070"
set lite        "\#e0d070"
set whitecolor  "\#ffffff"
set blackcolor  "\#000000"
set whiteborder "\#000000"
set blackborder "\#ffffff"
set highcolor   "\#b0d0e0"
set bestcolor   "\#bebebe"
set buttoncolor "\#b0c0d0"
set borderwidth 1

# Defaults for the PGN window:
# if ::pgn::showColor is 1, the PGN text will be colorized.
set ::pgn::showColor 1
set ::pgn::indentVars 1
set ::pgn::indentComments 1
set ::pgn::symbolicNags 1
set ::pgn::moveNumberSpaces 0
set ::pgn::shortHeader 0
set ::pgn::boldMainLine 1
set ::pgn::columnFormat 0
set ::pgn::stripMarks 0
set pgnColor(Header) "\#00008b"
set pgnColor(Main) "\#000000"
set pgnColor(Var) "\#0000ee"
set pgnColor(Nag) "\#ee0000"
set pgnColor(Comment) "\#008b00"
set pgnColor(Current) lightSteelBlue
set pgnColor(NextMove) "\#fefe80"
set pgnColor(Background) "\#ffffff"


# Defaults for initial directories:
set initialDir(base) "."
set initialDir(book) "."
set initialDir(epd) "."
set initialDir(html) "."
set initialDir(tex)  "."
set initialDir(report) "."
set initialDir(tablebase1) ""
set initialDir(tablebase2) ""
set initialDir(tablebase3) ""
set initialDir(tablebase4) ""
set initialDir(tablebase4) ""

# glistSize: Number of games displayed in the game list window
set glistSize 15

# glexport: Format for saving Game List to text file.
set glexportDefault "g6: w13 W4  b13 B4  r3:m2 y4 s11 o4"
set glexport $glexportDefault

# glistSelectPly: The number of moves to display in a game list entry
# when that entry is selected with button-2 to shoe the first moves
# of a game. E.g., a value of 4 might give: "1.e4 e5 2.Nf3 Nc6".
set glistSelectPly 80


# Default window locations:
foreach i {. .pgnWin .helpWin .crosstabWin .treeWin .commentWin .glist \
             .playerInfoWin .baseWin .treeBest .treeGraph .tourney .finder \
             .ecograph .statsWin .glistWin .maintWin .nedit} {
  set winX($i) -1
  set winY($i) -1
}

# Default PGN window size:
set winWidth(.pgnWin)  65
set winHeight(.pgnWin) 20

# Default help window size:
set winWidth(.helpWin)  50
set winHeight(.helpWin) 32

# Default stats window size:
set winWidth(.statsWin) 60
set winHeight(.statsWin) 13

# Default crosstable window size:
set winWidth(.crosstabWin)  65
set winHeight(.crosstabWin) 15

# Default tree window size:
set winWidth(.treeWin)  58
set winHeight(.treeWin) 20

# Default comment editor size:
set winWidth(.commentWin)  40
set winHeight(.commentWin)  6

# Default spellcheck results window size:
set winWidth(.spellcheckWin)  55
set winHeight(.spellcheckWin) 25

# Default player info window size:
set winWidth(.playerInfoWin)  45
set winHeight(.playerInfoWin) 20

# Default switcher window size:
set winWidth(.baseWin) 310
set winHeight(.baseWin) 110

# Default stats window lines:
array set ::windows::stats::display {
  r3000 1
  r2900 1
  r2800 1
  r2700 1
  r2600 1
  r2500 1
  r2400 1
  r2300 1
  r2200 0
  r2100 0
  r2000 0
  r1900 0
  r1800 0
  r1700 0
  r1600 0
  r1500 0
  r1400 0
  r1300 0
  r1200 0
  r1100 0
  r1000 0
  y1900 0
  y1950 0
  y1960 0
  y1970 0
  y1980 0
  y1990 0
  y1992 0
  y1994 0
  y1996 0
  y1998 1
  y2000 1
  y2001 1
  y2002 1
  y2003 1
  y2004 1
  y2005 1
  y2006 1
  y2007 1
  y2008 0
  y2009 0
  y2010 0
  y2011 0
  y2012 0
}

# Default PGN display options:
set pgnStyle(Tags) 1
set pgnStyle(Comments) 1
set pgnStyle(Vars) 1


# Default Tree sort method:
set tree(order) frequency

# Auto-save tree cache when closing tree window:
set tree(autoSave) 0

# Auto-save options when exiting:
set optionsAutoSave 1

#  Numeric locale: first char is decimal, second is thousands.
#  Example: ".," for "1,234.5" format; ",." for "1.234,5" format.
set locale(numeric) ".,"

# Ask before replacing existing moves: on by default
set askToReplaceMoves 1

# Show suggested moves: on by default
set suggestMoves 1

# Keyboard Move entry options:
set moveEntry(On) 1
set moveEntry(AutoExpand) 0
set moveEntry(Coord) 1

# Autoplay and animation delays in milliseconds:
set autoplayDelay 5000
set animateDelay 200

# Blunder Threshold
set mistakeThreshold 1.0
set blunderThreshold 3.0
set stopBlunderAnalysisAfter 10.0

# Geometry of windows:
array set geometry {}

# startup:
#   Stores which windows should be opened on startup.
set startup(pgn) 0
set startup(switcher) 0
set startup(tip) 1
set startup(tree) 0
set startup(finder) 0
set startup(crosstable) 0
set startup(gamelist) 0
set startup(stats) 0

# glistFields: Layout of the GameList window fields.
#    element 0: code (e.g. g for gameNumber, w for White name)
#    element 1: initial width, in characters
#    element 2: justification (left or right)
#    element 3: color
#    element 4: true if a separator field should follow
#
#    Note that the "g" (game number) field MUST appear somewhere,
#    but the fields can be in any order.
#    See the comments at the start of the function "PrintGameInfo" in
#    src/index.cpp for a list of available field codes.
#
set glistFields {
  { g  7 right black      1 }
  { w 14 left  darkBlue   0 }
  { W  5 right darkGreen  1 }
  { b 14 left  darkBlue   0 }
  { B  5 right darkGreen  1 }
  { e 10 left  black      0 }
  { s 10 left  black      0 }
  { n  2 right black      1 }
  { d  7 left  darkRed    1 }
  { r  3 left  blue       0 }
  { m  3 right black      1 }
  { o  5 left  darkGreen  0 }
  { O  6 left  darkGreen  1 }
  { D  1 left  darkRed    0 }
  { U  2 left  blue       1 }
  { V  2 right blue       0 }
  { C  2 right blue       0 }
  { A  2 right blue       0 }
  { S  1 left  darkRed    0 }
}

set glistDefaultFields $glistFields
set glistAllFields $glistFields
lappend glistAllFields { c  3 left  black      0 }
lappend glistAllFields { E  7 left  darkRed    0 }
lappend glistAllFields { F  7 left  darkBlue   0 }

# myPlayerNames:
#   List of player name patterns for which the chessboard should be
#   flipped each time a game is loaded to show the board from that
#   players perspective.
#
set myPlayerNames {}


# Game information area options:
set gameInfo(photos) 1
set gameInfo(hideNextMove) 0
set gameInfo(showMaterial) 0
set gameInfo(showFEN) 0
set gameInfo(showMarks) 1
set gameInfo(wrap) 0
set gameInfo(fullComment) 0
set gameInfo(showTB) 0
if {[sc_info tb]} { set gameInfo(showTB) 2 }

# Twin deletion options:

array set twinSettings {
  players No
  colors  No
  event   No
  site    Yes
  round   Yes
  year    Yes
  month   Yes
  day     No
  result  No
  eco     No
  moves   Yes
  skipshort  Yes
  setfilter  Yes
  undelete   Yes
  comments   Yes
  variations Yes
  usefilter  No
  delete     Shorter
}
array set twinSettingsDefaults [array get twinSettings]

# Opening report options:
array set optable {
  Stats 1
  Oldest 5
  Newest 5
  Popular 1
  MostFrequent 6
  MostFrequentWhite 1
  MostFrequentBlack 1
  AvgPerf 1
  HighRating 8
  Results 1
  Shortest 5
  ShortestWhite 1
  ShortestBlack 1
  MoveOrders 8
  MovesFrom 1
  Themes 1
  Endgames 1
  MaxGames 500
  ExtraMoves 1
}
array set optableDefaults [array get optable]

# Player report options
array set preport {
  Stats 1
  Oldest 5
  Newest 5
  MostFrequentOpponents 6
  AvgPerf 1
  HighRating 8
  Results 1
  MostFrequentEcoCodes 6
  Themes 1
  Endgames 1
  MaxGames 500
  ExtraMoves 1
}
array set preportDefaults [array get preport]

# Export file options:
set exportFlags(comments) 1
set exportFlags(indentc) 0
set exportFlags(vars) 1
set exportFlags(indentv) 1
set exportFlags(column) 0
set exportFlags(append) 0
set exportFlags(symbols) 1
set exportFlags(htmldiag) 0
set exportFlags(stripMarks) 0
set exportFlags(convertNullMoves) 1
set default_exportStartFile(PGN) {}
set default_exportEndFile(PGN) {}

set default_exportStartFile(LaTeX) {\documentclass[10pt,twocolumn]{article}
% This is a LaTeX file generated by ChessDB.
% You must have the "chess12" package installed to typeset this file.

\usepackage{times}
\usepackage{a4wide}
\usepackage{chess}
\usepackage[T1]{fontenc}

\setlength{\columnsep}{7mm}
\setlength{\parindent}{0pt}

% Macros for variations and diagrams:
\newenvironment{variation}{\begin{quote}}{\end{quote}}
\newenvironment{diagram}{\begin{nochess}}{$$\showboard$$\end{nochess}}

\begin{document}
}
set default_exportEndFile(LaTeX) {\end{document}
}


set default_exportStartFile(HTML) {<html>
<head><title>ChessDB export</title></head>
<body bgcolor="#ffffff">
}
set default_exportEndFile(HTML) {</body>
</html>
}

foreach type {PGN HTML LaTeX} {
  set exportStartFile($type) $default_exportStartFile($type)
  set exportEndFile($type) $default_exportEndFile($type)
}


# ::windows::switcher::vertical
#
#   If 1, Switcher frames are arranged vertically.
#
set ::windows::switcher::vertical 0
set ::windows::switcher::icons 1

# autoRaise: defines whether the "raise" command should be used to raise
# certain windows (like progress bars) when they become obscured.
# Some Unix window managers (e.g. some versions of Enlightenment and sawfish,
# so I have heard) have a bug where the Tcl/Tk "raise" command times out
# and takes a few seconds. Setting autoRaise to 0 will help avoid this.

set autoRaise 1

proc raiseWin {w} {
  global autoRaise
  if {$autoRaise} { raise $w }
  return
}

# autoIconify:
#   Specified whether ChessDB should iconify all other ChessDB windows when
#   the main window is iconified. Most people like this behaviour but
#   some window managers send an "UnMap" event when the user switches
#   to another virtual window without iconifying the ChessDB window so
#   users of such managers will probably want to turn this off.

set autoIconify 1


# Email configuration:
set email(logfile) [file join $chessdbLogDir "chessdbmail.log"]
set email(oldlogfile) [file join $chessdbUserDir "chessdbmail.log"]
set email(smtp) 1
set email(smproc) "/usr/lib/sendmail"
set email(server) localhost
set email(from) ""
set email(bcc) ""


### Audio move announcement options:

set ::utils::sound::announceNew 0
set ::utils::sound::announceForward 0
set ::utils::sound::announceBack 0

### Download and proxy options
set ::utils::download::useProxy 0
set ::utils::download::proxyHost ""
set ::utils::download::proxyPort ""
set ::utils::download::ICC_server chessclub.com
set ::utils::download::ICC_user ""
set ::utils::download::ICC_pass ""
set ::utils::download::ICC_port 5000
set ::utils::download::ICC_savepass 0
set ::utils::download::FICS_server freechess.org
set ::utils::download::FICS_user "guest" 
set ::utils::download::FICS_pass ""
set ::utils::download::FICS_port 5000 
set ::utils::download::FICS_savepass 0

### TWIC download window
set ::tools::twic::createLargePGN 0
set ::tools::twic::importDB 1
set ::tools::twic::autofirst 0
set ::tools::twic::autolast 0

### Players whose games to download - these are well known players on ICC
set ::tools::downloadFromChessServer::ICC_players  "boffman dougmo"
set ::tools::downloadFromChessServer::FICS_players "ymulakaj namaikatikura"

# Spell-checking file: default is "ratings.ssp".
# There are two spellcheck files - one has just the spellings, 
# the other spellings and ratings. But only ratings.ssp is distibuted. 
# with ChessDB. 

if {$windowsOS} {
  set ::utils::sound::soundFolder [file nativename [file join $::chessdbExeDir sounds]]
  set spellCheckFile [file nativename [file join $chessdbDataDir "ratings.ssp"] ]
} else {
  set spellCheckFile "/usr/local/share/chessdb/ratings.ssp"
  set ::utils::sound::soundFolder "/usr/local/share/chessdb/sounds"
}

# Engines data:
set engines(list) {}
set engines(sort) Time

# Set up ChessDB icon in Windows:
if {$::windowsOS} {
  # Look in each of the following directories for a file named chessdb.ico:
  set iconFileDirs [list \
    $chessdbExeDir $chessdbUserDir $chessdbConfigDir [file join $chessdbExeDir "src"]]

  foreach iconFileDir $iconFileDirs {
    set chessdbIconFile [file join $iconFileDir "chessdb.ico"]
    if {[file readable $chessdbIconFile]} {
      catch {wm iconbitmap . -default $chessdbIconFile}
    }
  }
}

# Add empty updateStatusBar proc to avoid errors caused by early
# closing of the splash window:
#
proc updateStatusBar {} {}

# Start up splash screen:

set ::splash::autoclose 0 
proc ::splash::make {} {
  wm withdraw .
  set w [toplevel .splash]
  wm withdraw $w
  wm protocol $w WM_DELETE_WINDOW [list wm withdraw $w]
  wm title $w "Welcome to ChessDB $::chessdbVersion"
  frame $w.f
  frame $w.b
  text $w.t -height 40 -width 70 -bg white -cursor top_left_arrow \
    -font font_Regular -wrap word \
    -yscrollcommand [list $w.ybar set] -setgrid 1 
  scrollbar $w.ybar -command [list $w.t yview]
  checkbutton $w.auto \
    -variable ::splash::autoclose -font font_Small -pady 5 -padx 5
  button $w.dismiss -width 8 -command [list wm withdraw $w] \
    -font font_Small
  pack $w.f -side top -expand yes -fill both
  pack $w.b -side top -fill x
  pack $w.auto -side left -in .splash.b -pady 2 -ipadx 10 -padx 10
  pack $w.dismiss -side right -in .splash.b -pady 2 -ipadx 10 -padx 10
  pack $w.ybar -in $w.f -side right -fill y
  pack $w.t -in $w.f -side left -fill both -expand yes

  # Centre the splash window:
  update idletasks
  set x [expr {[winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
                 - [winfo vrootx .]}]
  set y [expr {[winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
                 - [winfo vrooty .]}]
  wm geom $w +$x+$y
  wm deiconify $w

  bind $w <F1> {helpWindow Index}
  bind $w <Escape> {.splash.dismiss invoke}

#  $w.t tag configure red -fg darkRed
#  $w.t tag configure yellow -fg yellow
#  $w.t tag configure green -fg green
#  $w.t tag configure orange -fg orange
  $w.t tag configure indent -lmargin2 20
  $w.t tag configure h1 -font "Arial 17"
  $w.t tag configure h2 -font "Arial 14"
  $w.t tag configure h3 -font "Arial 12"
  $w.t tag configure underline -underline true
  $w.t tag configure fixed -font "Courier 14"
  $w.t image create end -image splash
}


# New ChessDB logo:
# Kindly designed by James Mossop, a professional designer, who
# gave his time for free. 
image create photo splash -format gif -data {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}

::splash::make

proc ::splash::add {text { mode black }} {
    if {! [winfo exists .splash]} {return}
    .splash.t configure -state normal
     if { $mode ==  "h1" } { 
      .splash.t insert end "\n$text"  h1 
     } elseif { $mode == "h2" } {
      .splash.t insert end "\n$text"  h2 
     } elseif { $mode == "h3" } {
      .splash.t insert end "\n$text"  h3 
     } elseif { $mode == "fixed" } {
      .splash.t insert end "\n$text"  fixed
     } elseif { $mode == "invert" } { 
      .splash.t insert end "\n$text"  invert
     } elseif { $mode == "black" } {
    .splash.t insert end "\n$text" indent 
     }
    .splash.t see end
    .splash.t configure -state disabled
    update
}

::splash::add "ChessDB $::chessdbVersion ($::chessdbVersionDate)\n" h1
::splash::add "http://chessdb.sourceforge.net/\n" fixed

# Remember old font settings before loading options file:
set fontOptions(oldRegular) $fontOptions(Regular)
set fontOptions(oldMenu) $fontOptions(Menu)
set fontOptions(oldSmall) $fontOptions(Small)
set fontOptions(oldTiny) $fontOptions(Tiny)
set fontOptions(oldFixed) $fontOptions(Fixed)

# New configuration file names:
set chessdbConfigFiles(options) "options.dat"
set chessdbConfigFiles(engines) "engines.dat"
set chessdbConfigFiles(engines.bak) "engines.dat"
set chessdbConfigFiles(recentfiles) "recent.dat"
set chessdbConfigFiles(history) "history.dat"
set chessdbConfigFiles(bookmarks) "bookmarks.dat"
set chessdbConfigFiles(reports) "reports.dat"

# chessdbConfigFile:
#   Returns the full path and name of a ChessDB configuration file,
#   given its configuration type.
#
proc chessdbConfigFile {type} {
  global chessdbConfigDir chessdbConfigFiles
  if {! [info exists chessdbConfigFiles($type)]} {
    return -code error "No such config file type: $type"
  }
  return [file nativename [file join $chessdbConfigDir $chessdbConfigFiles($type)]]
}


# Try to load saved options:
# Create user ".chessdb" directory in Unix if necessary:
# Since the options file used to be ".chessdb", rename it:
if {! [file isdirectory $chessdbUserDir]} {
  if {[file isfile $chessdbUserDir]} {
    catch {file rename -force $chessdbUserDir "$chessdbUserDir.old"}
  }
  if {[catch {file mkdir $chessdbUserDir} err]} {
    ::splash::add "Error creating ~/.chessdb directory: $err"
  } else {
    catch {file rename "$chessdbUserDir.old" $optionsFile}
  }
  # Rename old "~/.chessdb_sent_emails" if necessary:
  if {[file isfile [file nativename "~/.chessdb_sent_emails"]]} {
    catch {file rename [file nativename "~/.chessdb_sent_emails"] $email(logfile)}
  }
}

# Create the config, data, log & TWIC directories if they do not exist:
proc makeChessDBDir {dir} {
  if {! [file isdirectory $dir]} {
    if {[catch {file mkdir $dir} err]} {
      ::splash::add "Error creating directory $dir: $err"
    } else {
      ::splash::add "Created directory: $dir"
    }
  }
}

makeChessDBDir $chessdbConfigDir
makeChessDBDir $chessdbDataDir
makeChessDBDir $chessdbLogDir
makeChessDBDir $chessdbTWICDir

# Rename old email log file if necessary:
if {[file readable $email(oldlogfile)]  &&  ![file readable $email(logfile)]} {
  catch {file rename $email(oldlogfile) $email(logfile)}
}

# moveOldConfigFiles
#   Moves configuration files from the old Scid (3.4 and earlier) names
#   to the new file names used since Scid 3.5 and later as 
#   well as  all versions of ChessDB 
#
proc moveOldConfigFiles {} {
  global chessdbUserDir chessdbConfigDir
  foreach {oldname newname} {
    chessdbrc options.dat
    chessdb.opt options.dat
    chessdb.bkm bookmarks.dat
    chessdb.rfl recentfiles.dat
    engines.lis engines.dat
  } {
    set oldpath [file nativename [file join $chessdbUserDir $oldname]]
    set newpath [file nativename [file join $chessdbConfigDir $newname]]
    if {[file readable $oldpath]  &&  ![file readable $newpath]} {
      if {[catch {file rename $oldpath $newpath} err]} {
        ::splash::add "Error moving $oldpath to $newpath: $err"
      } else {
        ::splash::add "Moved old config file $oldname to $newpath"
      }
    }
  }
}

moveOldConfigFiles

set optionsFile [chessdbConfigFile options]


if {[catch {source $optionsFile} ]} {
  # ::splash::add "Unable to find the options file: \"$optionsFile\"  - one will be created for you."
   set ::OptionsFileStatus 0
} else {
  # ::splash::add "Your options file \"$optionsFile\" was found and loaded."
   set ::OptionsFileStatus 1
}

# Now, if the options file was written by ChessDB 3.5 or older, it has a lot of
# yucky variable names in the global namespace. So convert them to the new
# namespace variables:
#
proc ConvertOldOptionVariables {} {

  set oldNewNames {
    autoCloseSplash ::splash::autoclose
    switcherVertical ::windows::switcher::vertical
    doColorPgn ::pgn::showColor
    pgnIndentVars ::pgn::indentVars
    pgnIndentComments ::pgn::indentComments
    pgnShortHeader ::pgn::shortHeader
    pgnMoveFont ::pgn::boldMainLine
    pgnMoveNumSpace ::pgn::moveNumberSpaces
    pgnStripMarks ::pgn::stripMarks
    pgnSymbolicNags ::pgn::symbolicNags
    pgnColumn ::pgn::columnFormat
  }

  foreach {old new} $oldNewNames {
    if {[info exists ::$old]} {
      set $new [set ::$old]
    }
  }
}

ConvertOldOptionVariables


# Reconfigure fonts if necessary:
foreach i {Regular Menu Small Tiny Fixed} {
  if {$fontOptions($i) == $fontOptions(old$i)} {
    # Old font format in options file, or no file. Extract new options:
    set fontOptions($i) {}
    lappend fontOptions($i) [font actual font_$i -family]
    lappend fontOptions($i) [font actual font_$i -size]
    lappend fontOptions($i) [font actual font_$i -weight]
    lappend fontOptions($i) [font actual font_$i -slant]
  } else {
    # New font format in options file:
    configureFont $i
  }
}

# Check board size is valid:
set newSize [lindex $boardSizes 0]
foreach sz $boardSizes {
  if {$boardSize >= $sz} { set newSize $sz }
}
set boardSize $newSize

# Check for old (single-directory) tablebase option:
if {[info exists initialDir(tablebase)]} {
  set initialDir(tablebase1) $initialDir(tablebase)
}

# font_Regular is the default font for widgets:
option add *Font font_Regular

# Use font_Menu for menu entries:
option add *Menu*Font font_Menu
# option add *Menubutton*Font font_Menu

if {$unixOS} {
  option add Scrollbar*borderWidth 1
}

# Set the radiobutton and checkbutton background color if desired.
# I find the maroon color on Unix ugly!
if {$unixOS} {
  option add *Radiobutton*selectColor $buttoncolor
  option add *Checkbutton*selectColor $buttoncolor
  option add *Menu*selectColor $buttoncolor
}

set fontsize [font configure font_Regular -size]
set font [font configure font_Regular -family]

font create font_Bold -family $font -size $fontsize -weight bold
font create font_BoldItalic -family $font -size $fontsize -weight bold \
  -slant italic
font create font_Italic -family $font -size $fontsize -slant italic
font create font_H1 -family $font -size [expr {$fontsize + 8} ] -weight bold
font create font_H2 -family $font -size [expr {$fontsize + 6} ] -weight bold
font create font_H3 -family $font -size [expr {$fontsize + 4} ] -weight bold
font create font_H4 -family $font -size [expr {$fontsize + 2} ] -weight bold
font create font_H5 -family $font -size [expr {$fontsize + 0} ] -weight bold

set fontsize [font configure font_Small -size]
set font [font configure font_Small -family]
font create font_SmallBold -family $font -size $fontsize -weight bold
font create font_SmallItalic -family $font -size $fontsize -slant italic

# Start in the clipbase, if no database is loaded at startup.
sc_base switch clipbase

###
### End of file: start.tcl
